@@ -36,7 +36,10 @@ impl TreeSitterParser {
                 .cpp(true)
                 .flag_if_supported("-Wno-implicit-fallthrough")
                 .flag_if_supported("-Wno-unused-parameter")
-                .flag_if_supported("-Wno-ignored-qualifiers");
+                .flag_if_supported("-Wno-ignored-qualifiers")
+                // Workaround for: https://github.com/ganezdragon/tree-sitter-perl/issues/16
+                // should be removed after fixed.
+                .flag_if_supported("-Wno-return-type");
 
             if cfg!(windows) {
                 cpp_build.flag("/std:c++14");
@@ -124,6 +127,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-haskell-src",
             extra_files: vec!["scanner.c"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-hcl",
+            src_dir: "vendor/tree-sitter-hcl-src",
+            extra_files: vec!["scanner.cc"],
+        },
         TreeSitterParser {
             name: "tree-sitter-janet-simple",
             src_dir: "vendor/tree-sitter-janet-simple-src",
@@ -174,6 +182,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-php-src",
             extra_files: vec!["scanner.cc"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-perl",
+            src_dir: "vendor/tree-sitter-perl-src",
+            extra_files: vec!["scanner.cc"],
+        },
         TreeSitterParser {
             name: "tree-sitter-python",
             src_dir: "vendor/tree-sitter-python-src",
@@ -194,6 +207,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-scala-src",
             extra_files: vec!["scanner.c"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-swift",
+            src_dir: "vendor/tree-sitter-swift-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-toml",
             src_dir: "vendor/tree-sitter-toml-src",
