@@ -1,4 +1,4 @@
-use std::{borrow::Borrow, env};
+use std::{borrow::Borrow, env, ffi::OsString};
 
 use atty::Stream;
 use clap::{crate_authors, crate_description, crate_version, Arg, Command};
@@ -8,16 +8,28 @@ use crate::{guess_language, style::BackgroundColor};
 
 pub const DEFAULT_NODE_LIMIT: u32 = 30_000;
 pub const DEFAULT_BYTE_LIMIT: usize = 1_000_000;
+pub const DEFAULT_TAB_WIDTH: usize = 8;
 
 const USAGE: &str = concat!(env!("CARGO_BIN_NAME"), " [OPTIONS] OLD-PATH NEW-PATH");
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub enum ColorOutput {
     Always,
     Auto,
     Never,
 }
 
+#[derive(Debug, Clone)]
+pub struct DisplayOptions {
+    pub background_color: BackgroundColor,
+    pub use_color: bool,
+    pub display_mode: DisplayMode,
+    pub print_unchanged: bool,
+    pub tab_width: usize,
+    pub display_width: usize,
+    pub syntax_highlight: bool,
+}
+
 fn app() -> clap::Command<'static> {
     Command::new("Difftastic")
         .override_usage(USAGE)
@@ -64,6 +76,17 @@ fn app() -> clap::Command<'static> {
                 .validator(|s| s.parse::<usize>())
                 .required(false),
         )
+        .arg(
+            Arg::new("tab-width")
+                .long("tab-width")
+                .takes_value(true)
+                .value_name("NUM_SPACES")
+                .long_help("Treat a tab as this many spaces.")
+                .env("DFT_TAB_WIDTH")
+                .default_value(formatcp!("{}", DEFAULT_TAB_WIDTH))
+                .validator(|s| s.parse::<usize>())
+                .required(false),
+        )
         .arg(
             Arg::new("display").long("display")
                 .possible_values(["side-by-side", "side-by-side-show-both", "inline", ])
@@ -84,13 +107,21 @@ fn app() -> clap::Command<'static> {
                 .possible_values(["dark", "light"])
                 .help("Set the background brightness. Difftastic will prefer brighter colours on dark backgrounds.")
         )
+        .arg(
+            Arg::new("syntax-highlight").long("syntax-highlight")
+                .value_name("ON/OFF")
+                .env("DFT_SYNTAX_HIGHLIGHT")
+                .possible_values(["on", "off"])
+                .default_value("on")
+                .help("Enable or disable syntax highlighting.")
+        )
         .arg(
             Arg::new("skip-unchanged").long("skip-unchanged")
                 .help("Don't display anything if a file is unchanged.")
         )
         .arg(
             Arg::new("missing-as-empty").long("missing-as-empty")
-                .help("Treat paths that don't exist as equivalent to an empty file.")
+                .help("Treat paths that don't exist as equivalent to an empty file. Only applies when diffing files, not directories.")
         )
         .arg(
             Arg::new("language").long("language")
@@ -140,16 +171,12 @@ pub enum Mode {
     Diff {
         node_limit: u32,
         byte_limit: usize,
-        print_unchanged: bool,
+        display_options: DisplayOptions,
         missing_as_empty: bool,
-        display_mode: DisplayMode,
-        background_color: BackgroundColor,
-        color_output: ColorOutput,
-        display_width: usize,
         display_path: String,
         language_override: Option<guess_language::Language>,
-        lhs_path: String,
-        rhs_path: String,
+        lhs_path: OsString,
+        rhs_path: OsString,
     },
     DumpTreeSitter {
         path: String,
@@ -194,23 +221,22 @@ pub fn parse_args() -> Mode {
         };
     }
 
-    // TODO: Use OsString rather than forcing paths to be valid UTF-8.
-    let args: Vec<_> = matches.values_of_lossy("paths").unwrap_or_default();
+    let args: Vec<_> = matches.values_of_os("paths").unwrap_or_default().collect();
     info!("CLI arguments: {:?}", args);
 
     // TODO: document these different ways of calling difftastic.
     let (display_path, lhs_path, rhs_path) = match &args[..] {
         [lhs_path, rhs_path] => (
-            rhs_path.to_string(),
-            lhs_path.to_string(),
-            rhs_path.to_string(),
+            rhs_path.to_owned(),
+            lhs_path.to_owned(),
+            rhs_path.to_owned(),
         ),
         [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
             // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
             (
-                display_path.to_string(),
-                lhs_tmp_file.to_string(),
-                rhs_tmp_file.to_string(),
+                display_path.to_owned(),
+                lhs_tmp_file.to_owned(),
+                rhs_tmp_file.to_owned(),
             )
         }
         [_old_name, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode, new_name, _similarity] =>
@@ -219,9 +245,9 @@ pub fn parse_args() -> Mode {
             // TODO: mention old name as well as diffing.
             // TODO: where does git document these 9 arguments?
             (
-                new_name.to_string(),
-                lhs_tmp_file.to_string(),
-                rhs_tmp_file.to_string(),
+                new_name.to_owned(),
+                lhs_tmp_file.to_owned(),
+                rhs_tmp_file.to_owned(),
             )
         }
         _ => {
@@ -281,6 +307,8 @@ pub fn parse_args() -> Mode {
         BackgroundColor::Dark
     };
 
+    let syntax_highlight = matches.value_of("syntax-highlight") == Some("on");
+
     let node_limit = matches
         .value_of("node-limit")
         .expect("Always present as we've given clap a default")
@@ -293,22 +321,36 @@ pub fn parse_args() -> Mode {
         .parse::<usize>()
         .expect("Value already validated by clap");
 
+    let tab_width = matches
+        .value_of("tab-width")
+        .expect("Always present as we've given clap a default")
+        .parse::<usize>()
+        .expect("Value already validated by clap");
+
     let print_unchanged = !matches.is_present("skip-unchanged");
     let missing_as_empty = matches.is_present("missing-as-empty");
 
+    let use_color = should_use_color(color_output);
+
+    let display_options = DisplayOptions {
+        background_color,
+        use_color,
+        print_unchanged,
+        tab_width,
+        display_mode,
+        display_width,
+        syntax_highlight,
+    };
+
     Mode::Diff {
         node_limit,
         byte_limit,
-        print_unchanged,
+        display_options,
         missing_as_empty,
-        display_mode,
-        background_color,
-        color_output,
-        display_width,
-        display_path,
+        display_path: display_path.to_string_lossy().to_string(),
         language_override,
-        lhs_path,
-        rhs_path,
+        lhs_path: lhs_path.to_owned(),
+        rhs_path: rhs_path.to_owned(),
     }
 }
 
